/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import java.io.*;
import java.util.*;

import javax.swing.text.*;

public class SimpleElement extends HtmlElement {
    String name;
    AttributeSet set;

    public SimpleElement(String name, AttributeSet set) {
        this.name = name;
        this.set = set.copyAttributes();
    }

    @Override
    protected void writeToStringOn(Writer writer) throws IOException {
        writer.write('<');
        writer.write(name);

        writeToStringAttributesOn(writer);
        writer.write('>');
        this.writeToStringContentOn(writer);
        writer.write("</");
        writer.write(name);
        writer.write('>');
    }

    protected void writeToStringAttributesOn(Writer writer) throws IOException {
        Object attributeName;
        boolean first = true;
        if (this.set.getAttributeCount() > 0)
            writer.write(' ');
        for (Enumeration names = this.set.getAttributeNames(); names.hasMoreElements();) {
            if (!first) {
                writer.write(' ');
            }
            first = false;
            attributeName = names.nextElement();
            writer.write(attributeName.toString());
            writer.write("=\"");
            writer.write(this.set.getAttribute(attributeName).toString());
            writer.write("\"");
        }
    }

    protected void writeToStringContentOn(Writer writer) throws IOException {
    }

    @Override
    public Object getAttribute(Object key) {
        return set.getAttribute(key);
    }

}
