/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import java.io.*;
import java.util.*;

import javax.swing.text.*;

public class ComplexElement extends SimpleElement {
    List<HtmlElement> children;

    public ComplexElement(String name, AttributeSet set) {
        super(name, set);
        this.children = new ArrayList<HtmlElement>();
    }

    public List<HtmlElement> getChildren() {
        return Collections.unmodifiableList(children);
    }

    @Override
    protected void addChild(HtmlElement anElement) {
        children.add(anElement);
    }

    @Override
    protected void writeToStringContentOn(Writer writer) throws IOException {
        for (HtmlElement each : children) {
            each.writeToStringOn(writer);
        }
    }

    @Override
    protected void writeTextOn(Writer writer) throws IOException {
        for (HtmlElement element : children)
            element.writeTextOn(writer);
    }
}
