/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import java.util.*;

import javax.swing.text.*;
import javax.swing.text.html.HTML.*;
import javax.swing.text.html.HTMLEditorKit.*;

class HtmlCallback extends ParserCallback {
    Stack<Info> stack;
    HtmlRule startRule;
    HtmlRule satisfiedRule;
    HtmlElement currentElement;
    List<HtmlElement> found;
    AttributeSet currentAttributes;
    Tag currentTag;

    class Info {
        HtmlRule satisfied;
        HtmlElement element;
        Tag tag;
        AttributeSet attributes;

        public Info() {
            this.satisfied = HtmlCallback.this.satisfiedRule;
            this.element = HtmlCallback.this.currentElement;
            this.attributes = HtmlCallback.this.currentAttributes;
            this.tag = HtmlCallback.this.currentTag;
        }

        public void rollback() {
            HtmlCallback.this.currentAttributes = this.attributes;
            HtmlCallback.this.currentTag = this.tag;

            if (this.element == null) {
                if (HtmlCallback.this.currentElement != null) {
                    HtmlCallback.this.found.add(HtmlCallback.this.currentElement);
                }
            } else {
                this.element.addChild(HtmlCallback.this.currentElement);
            }
            HtmlCallback.this.satisfiedRule = this.satisfied;
            HtmlCallback.this.currentElement = this.element;
        }
    }

    /**
     * 
     */
    public HtmlCallback(HtmlRule startRule) {
        super();
        stack = new Stack<Info>();
        this.satisfiedRule = null;
        this.startRule = startRule;
        this.currentElement = null;
        this.found = new ArrayList<HtmlElement>();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#flush()
     */
    @Override
    public void flush() throws BadLocationException {

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleComment(char[],
     *      int)
     */
    @Override
    public void handleComment(char[] data, int pos) {

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleEndOfLineString(java.lang.String)
     */
    @Override
    public void handleEndOfLineString(String eol) {
        if (currentElement != null)
            currentElement.addChild(new TextElement(eol));

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleEndTag(javax.swing.text.html.HTML.Tag,
     *      int)
     */
    @Override
    public void handleEndTag(Tag tag, int pos) {
        stack.pop().rollback();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleError(java.lang.String,
     *      int)
     */
    @Override
    public void handleError(String errorMsg, int pos) {

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleSimpleTag(javax.swing.text.html.HTML.Tag,
     *      javax.swing.text.MutableAttributeSet, int)
     */
    @Override
    public void handleSimpleTag(Tag tag, MutableAttributeSet attributes, int pos) {
        if (currentElement != null) {
            currentElement.addChild(new ComplexElement(tag.toString(), attributes));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleStartTag(javax.swing.text.html.HTML.Tag,
     *      javax.swing.text.MutableAttributeSet, int)
     */
    @Override
    public void handleStartTag(Tag tag, MutableAttributeSet attributes, int pos) {
        currentTag = tag;
        currentAttributes = attributes.copyAttributes();

        stack.push(new Info());
        checkRules(tag, attributes, null);
    }

    protected void checkRules(Tag tag, AttributeSet attributes, char[] text) {
        HtmlRule searchRule;

        if (satisfiedRule == null) {
            if (startRule.satisfiedBy(tag, attributes, text)) {
                satisfiedRule = startRule;
            }
        } else if ((searchRule = satisfiedRule.getNext()) != null && searchRule.satisfiedBy(tag, attributes, text)) {
            satisfiedRule = searchRule;
        }
        if (satisfiedRule != null && satisfiedRule.shouldFire()) {
            if (currentElement == null || text == null) {
                currentElement = new ComplexElement(tag.toString(), attributes);
            }
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.text.html.HTMLEditorKit.ParserCallback#handleText(char[],
     *      int)
     */
    @Override
    public void handleText(char[] data, int pos) {
        checkRules(currentTag, currentAttributes, data);
        if (currentElement != null)
            currentElement.addChild(new TextElement(new String(data)));
    }

    public List<HtmlElement> getFound() {
        return Collections.unmodifiableList(found);
    }
}