/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import javax.swing.text.*;
import javax.swing.text.html.HTML.*;

public abstract class HtmlRule {
    HtmlRule next;

    public static HtmlRule onTags(Tag... tags) {
        HtmlRule result = null;
        for (int i = 0; i < tags.length; i++) {
            if (result == null) {
                result = TagRule.on(tags[i]);
            } else {
                result.thenTag(tags[i]);
            }
        }
        return result;
    }

    public static HtmlRule onAttributes(String[]... attributes) {
        HtmlRule result = null;
        for (int i = 0; i < attributes.length; i++) {
            if (result == null) {
                result = AttributeRule.on(attributes[i][0], attributes[i][1]);
            } else {
                result.thenAttribute(attributes[i][0], attributes[i][1]);
            }
        }
        return result;
    }

    public static HtmlRule andAll(HtmlRule... htmlRules) {
        CompoundRule result = new AndRule();
        for (HtmlRule each : htmlRules)
            result.addChild(each);
        return result;
    }

    public static HtmlRule orAll(HtmlRule... htmlRules) {
        CompoundRule result = new OrRule();
        for (HtmlRule each : htmlRules)
            result.addChild(each);
        return result;
    }

    public HtmlRule getNext() {
        return next;
    }

    public boolean shouldFire() {
        return next == null;
    }

    public HtmlRule then(HtmlRule aRule) {
        if (next != null)
            return next.then(aRule);
        next = aRule;
        return next;
    }

    public HtmlRule thenAnd(HtmlRule... htmlRules) {
        return then(andAll(htmlRules));
    }

    public HtmlRule thenOr(HtmlRule... htmlRules) {
        return then(orAll(htmlRules));
    }

    public HtmlRule thenTag(Tag tag) {
        return then(TagRule.on(tag));
    }

    public HtmlRule thenAttribute(String name, String value) {
        return then(AttributeRule.on(name, value));
    }

    public abstract boolean satisfiedBy(Tag tag, AttributeSet attributes, char[] text);
}
