/**
 * Copyright (c) 2005 Blaine Buxton
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.btb.needle;

import java.util.*;

import javax.swing.text.*;
import javax.swing.text.html.HTML.*;

public class AttributeRule extends HtmlRule {
    Object name;
    Object value;

    public static AttributeRule on(Object name, Object value) {
        return new AttributeRule(name, value);
    }

    AttributeRule(Object name, Object value) {
        this.name = name;
        this.value = value;
    }

    @Override
    @SuppressWarnings("unused")
    public boolean satisfiedBy(Tag tag, AttributeSet attributes, char[] text) {
        Enumeration enumeration = attributes.getAttributeNames();
        while (enumeration.hasMoreElements()) {
            Object key = enumeration.nextElement();
            Object entry = attributes.getAttribute(key);
            if (entry.equals(value) || entry.toString().matches(value.toString()))
                return true;

        }
        return false;
    }

    public String toString() {
        return name + "=" + value;
    }
}
